/*
 * Created on Nov 3, 2004
 *
 */
package com.sunlabs.archsim.engine;





import com.sunlabs.archsim.base.simulation.Event;

import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 */
public class EventHeapClassTest extends TestCase {

	/*
	 * Class under test for void EventHeap()
	 */
	public void testEventHeap() {
		EventHeap h= new EventHeap();
		assertEquals(h.size(), 0);
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
	} // test default constructor for EventHeap

	public static void main(String[] args) {
		TestRunner.run(EventHeapClassTest.class);
	}
	
	/*
	 * Class under test for void EventHeap(int)
	 */
	public void testEventHeapint() {
		int bigSize= EventHeap.DefaultCapacity + 50;
		EventHeap h= new EventHeap(bigSize);
		assertEquals(h.size(), 0);
		assertEquals(h.capacity(), bigSize);
		
		int tooSmall= -50;
		h= new EventHeap(tooSmall);
		assertEquals(h.size(), 0);
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);		
	} // test EventHeap(int) constuctor

	public void testSize() {
		EventHeap h= new EventHeap();
		assertEquals(h.size(), 0);
		
		h= new EventHeap(-50);
		assertEquals(h.size(), 0);
		
		h= new EventHeap(EventHeap.DefaultCapacity*2);
		assertEquals(h.size(), 0);
	
	} // testSize

	public void testCapacity() {
		EventHeap h= new EventHeap();
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
		
		h= new EventHeap(-50);
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
		
		int bigSize= EventHeap.DefaultCapacity*2;
		h= new EventHeap(bigSize);
		assertEquals(h.capacity(), bigSize);
	} // testCapacity

	public void testResize() {
		EventHeap h= new EventHeap();
		int s= h.resize(EventHeap.DefaultCapacity*2);
		assertEquals(s, EventHeap.DefaultCapacity*2);
		assertEquals(h.event.length, EventHeap.DefaultCapacity*2);
		
		// decrease aray size */
		h= new EventHeap(EventHeap.DefaultCapacity*2);
		s= h.resize(EventHeap.DefaultCapacity + 10);
		assertEquals(s, EventHeap.DefaultCapacity + 10);
		assertEquals(h.event.length, EventHeap.DefaultCapacity + 10);
		
		// try to decrease array size too much
		int startingSize= EventHeap.DefaultCapacity * 2;
		h= new EventHeap(startingSize);
		s= h.resize(EventHeap.DefaultCapacity - 10);

		assertEquals(s, startingSize);
		assertEquals(h.event.length, startingSize);
		
	}

	public void testParentint() {
		EventHeap h= new EventHeap();
		int p= h.parent(0);
		assertEquals(p,0);
		
		p= h.parent(1);
		assertEquals(p, 0);
		
		p= h.parent(2);
		assertEquals(p, 0);
		
		p= h.parent(9);
		assertEquals(p, 4);
		
		p= h.parent(10);
		assertEquals(p, 4);
	} // testParent
	
	public void testLeftChildint() {
		EventHeap h= new EventHeap();
		
		int l= h.leftChild(0);
		assertEquals(l,1);
		
		l= h.leftChild(6);
		assertEquals(l,13);
	} // testLeftChild
	
	public void testRightChildint() {
		EventHeap h= new EventHeap();
		
		int r= h.rightChild(0);
		assertEquals(r,2);
		
		r= h.rightChild(6);
		assertEquals(r,14);
		
	} // testRightChild
	
	public void testGetTop() {
		EventHeap h= new EventHeap(); // the heap to test inserts
				
		// build the heap, remember the events
		Event e100= new Event(100);
		h.insert(e100); 
		Event e200= new Event(200);
		h.insert(e200);
		Event e50= new Event(50);
		h.insert(e50);
		Event e1000= new Event(1000);
		h.insert(e1000);
		Event e25= new Event(25);
		h.insert(e25);
		Event e75= new Event(75);
		h.insert(e75);
		
		// remove the top event and verify that the heap is correct
		Event top= h.getTop();
		assertEquals(top, e25);
		assertEquals(h.event[0], e50);
		assertEquals(h.event[1], e100);
		assertEquals(h.event[2], e75);
		assertEquals(h.event[3], e1000);
		assertEquals(h.event[4], e200);
		
		top= h.getTop();
		assertEquals(top, e50);
		assertEquals(h.event[0], e75);
		assertEquals(h.event[1], e100);
		assertEquals(h.event[2], e200);
		assertEquals(h.event[3], e1000);
		
		top= h.getTop();
		assertEquals(top, e75);
		assertEquals(h.event[0], e100);
		assertEquals(h.event[1], e1000);
		assertEquals(h.event[2], e200);

		top= h.getTop();
		assertEquals(top, e100);
		assertEquals(h.event[0], e200);
		assertEquals(h.event[1], e1000);

		top= h.getTop();
		assertEquals(top, e200);
		assertEquals(h.event[0], e1000);	

		top= h.getTop();
		assertEquals(top, e1000);
		assertEquals(h.event[0], null);
		
		// removing from an empty heap returns null
		top= h.getTop();
		assertEquals(top, null);
		assertEquals(h.nextEmptyPlace, 0);
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
		
		// does the capacity stay the same change after the heap has increased in size
		// and then the occupancy decreased below the original size
		h= new EventHeap();
		// fill up the queue
		for (int i= 0; i< EventHeap.DefaultCapacity; i++) {
			h.insert(new Event(i));
		}
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
		h.insert(new Event(1000));
		assertEquals(h.capacity(), 2*EventHeap.DefaultCapacity);
		// take a few items off the top of the heap, the capacity must stay the same
		for (int i= 0; i<10; i++) {
			top= h.getTop();
			assertEquals(h.capacity(), 2*EventHeap.DefaultCapacity);
		}
		
} // testGetTop

	public void testPeek() {
		EventHeap h= new EventHeap();
		
		Event e= new Event(100);
		h.insert(e); 
		Event p= h.peek();
		assertEquals(p, h.event[0]);
		
		e= new Event(200);
		h.insert(e);
		p= h.peek();
		assertEquals(p, h.event[0]);
		
		e= new Event(50);
		h.insert(e);
		p= h.peek();
		assertEquals(p, h.event[0]);
		
		e= new Event(1000);
		h.insert(e);
		p= h.peek();
		assertEquals(p, h.event[0]);
		
		e= new Event(25);
		h.insert(e);
		p= h.peek();
		assertEquals(p, h.event[0]);
		
		e= new Event(75);
		h.insert(e);
		p= h.peek();
		assertEquals(p, h.event[0]);
	} // testPeek

	public void testInsert() {
		EventHeap h= new EventHeap(); // the heap to test inserts
		
		Event refEvent[]= new Event[10]; // reference event array - to be compared with the result
		
		Event e= new Event(100);
		h.insert(e);    // insert the event on the heap
		refEvent[5]= e; // populate the reference array
		
		e= new Event(200);
		h.insert(e);
		refEvent[4]= e;
		
		e= new Event(50);
		h.insert(e);
		refEvent[1]= e;
		
		e= new Event(1000);
		h.insert(e);
		refEvent[3]= e;
		
		e= new Event(25);
		h.insert(e);
		refEvent[0]= e;
		
		e= new Event(75);
		h.insert(e);
		refEvent[2]= e;
		
		// compare the final event heap/array withih h with the reference event array
		assertEquals(refEvent[0], h.event[0]);
		assertEquals(refEvent[1], h.event[1]);
		assertEquals(refEvent[2], h.event[2]);
		assertEquals(refEvent[3], h.event[3]);
		assertEquals(refEvent[4], h.event[4]);
		assertEquals(refEvent[5], h.event[5]);
		
		// does the capacity change when the queue is filled to its capacity?
		h= new EventHeap();
		// fill up the queue
		for (int i= 0; i< EventHeap.DefaultCapacity; i++) {
			h.insert(new Event(i));
		}
		assertEquals(h.capacity(), EventHeap.DefaultCapacity);
		h.insert(new Event(1000));
		assertEquals(h.capacity(), 2*EventHeap.DefaultCapacity);

	} // testInsert

	public void testClear() {
		EventHeap h= new EventHeap(); // the heap to test inserts
		
		// build the heap, remember the events
		Event e= new Event(100);
		h.insert(e); 
		e= new Event(200);
		h.insert(e);
		e= new Event(50);
		h.insert(e);
		e= new Event(1000);
		h.insert(e);
		e= new Event(25);
		h.insert(e);
		e= new Event(75);
		h.insert(e);
		
		h.clear();
		assertEquals(h.nextEmptyPlace, 0);
		assertEquals(h.event[0], null);
	} // testClear
	
} // EventHeapTestClass
