/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.adder_oob;
import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;



/*
 * Copyright � 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */


/**
 * @author mh193489
 * 
 * Adder ship for fleet.
 * Accepts integer on In terminal and outputs the cumulative sum on Out terminal.
 */
public class Adder_oob extends Component {
		
	InputTerminal in1;
	OutputTerminal ackIn1;

    InputTerminal in2;
	OutputTerminal ackIn2;

	OutputTerminal out;
	InputTerminal ackOut;
        
	/** accumulator action hidden in the ship */
	AdderAction adder;
	
	public Adder_oob(String name, Parameters params) {
        super(name, null);

		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				// ignore all parametters
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");							
			} // for
		} // if
        
        construct(name);			
	} // Adder

	public Adder_oob(String name) {
    	super(name, null);
        construct(name);				
	} // Adder


public InputTerminal getIn1() {
return in1;
}
	
    private void construct(String name) {
		
        // create terminals for the Adder component
		in1= new InputTerminal(AdderDefaults.IN1, this);
        this.addMember(in1);

        ackIn1= new OutputTerminal(AdderDefaults.ACK_IN1, this);
        this.addMember(ackIn1);
		
        in2= new InputTerminal(AdderDefaults.IN2, this);
		this.addMember(in2);

        ackIn2= new OutputTerminal(AdderDefaults.ACK_IN2, this);
		this.addMember(ackIn2);

        out= new OutputTerminal(AdderDefaults.OUT, this);
		this.addMember(out);

        ackOut= new InputTerminal(AdderDefaults.ACK_OUT, this);
		this.addMember(ackOut);

        // create the action performing the adding
		adder= new AdderAction(AdderDefaults.ADDER, this);
		this.addMember(adder);

        // attach terminals to the action
        adder.attach(in1);
		adder.attach(ackIn1);
		adder.attach(in2);
		adder.attach(ackIn2);
		adder.attach(out);
		adder.attach(ackOut);

        }
        

	
	public OutputTerminal getAckIn1() {
		return ackIn1;
	}

        public InputTerminal getIn2() {
		return in2;
	}
	
	public OutputTerminal getAckIn2() {
		return ackIn2;
	}
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
	
	/** 
	 * Get a terminal by name.
	 * Return null if no such terminal exists.
	 */
	public Terminal getTerminal(String name) {
		if (name.equals(in1.getBaseName()) 
				|| name.equals(in1.getBaseAlias()))
			return in1;
		else if (name.equals(ackIn1.getBaseName()) 
				|| name.equals(ackIn1.getBaseAlias()))
			return ackIn1;
                else if (name.equals(in2.getBaseName()) 
				|| name.equals(in2.getBaseAlias()))
			return in2;
		else if (name.equals(ackIn2.getBaseName()) 
				|| name.equals(ackIn2.getBaseAlias()))
			return ackIn2;
		else if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else 
			return null;
			
	} // getTerminal
	
} // Adder;


