/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */
/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.testing;


import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;

import com.sunlabs.archsim.globals.GlobalDefaults;

/**
 * @author mh193489
 *
 */
public class IntegerSequenceSource extends Component {

	OutputTerminal out;
	InputTerminal ackOut;        
    
        Action action;

    	public IntegerSequenceSource(String name, Parameters params) {
            super(name, null);
            
    		// process the parameters
    		if (params != null) {
    			for (int i= 0; i<params.size(); i++) {
    				String pName= params.getName(i);
    				String sValue= params.getValue(i);
    		
    				director.error("Component: " + name + ", id= " + getID() 
    						+ ": Attribute " + pName + " with value " + sValue
    						+ " is ignored");					
    		
    			} // for
    		} // if params != null
    		
    		construct(name, 0);
    	}
    		        
        
    	
    	public IntegerSequenceSource(String name) {
        	super(name, null);
            construct(name, 0);				
    	} // IntegerSequenceSource

     
        public void construct(String name, int start) {
            // create terminals
            out = new OutputTerminal(TestingDefaults.OUT, this);
            ackOut = new InputTerminal(TestingDefaults.ACK_OUT, this);

            // create action
            action = new IntegerSequenceAction(name + "Action", start);
            
            // attach terminals to action
            action.attach(out);
            action.attach(ackOut);
                                
        } // construct

	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
        
        public Terminal getTerminal(String name) {
                if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else 
			return null;
			
	} // getTerminal        


        private class IntegerSequenceAction extends Action {

                Function out= null;

                public IntegerSequenceAction(String name, int start) {
                        super(name);
                        out= new GenerateData(start);
                        outputFunctions.put(TestingDefaults.OUT, out);
                } // IntegerSequenceAction

                /** Generate new data items */
                private class GenerateData extends Function {

                        private int last;
                        private static final int MAX= 128;
                        
                        public GenerateData(int start) {
                                super("GenerateData");
                                this.last= start - 1;
                                functionDelay= TestingDefaults.ACTION_DELAY;
                        }

                        public Object value() {
                                // prepare payload for output
                                Payload payload= new Payload();
                                
                                // get the next number
                                last++; 
                                // get the following number, if not overflow
                                if (MAX <= last) {
                                        payload.data= new Data(last);
                                        
                                } else {
                                        payload.data= new Data(false);
                                    
                                }
                                
                                // log to journal
                                if (GlobalDefaults.JOURNAL_ACTION_DEF) {
                                        director.journal("GenerateData " + IntegerSequenceAction.this.getName() 
                                                + ", id= " + IntegerSequenceAction.this.getID()
                                                + ", generated " + last);
                                }
                                // wrap last in Payload 
                                //Data data= new Data(last);
                                
                                return payload;	
                        } // value
                } // GenerateData
        }

}

