/*
 * Copyright ??? 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *
 */

/*
 * Created on October 26, 2005, 4:58 PM
 */

package com.sunlabs.archsim.fleet.ships.tokenaccumulator;


import java.util.List;
import java.util.Iterator;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;


import com.sunlabs.archsim.fleet.ships.testing.DataSink;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;

import com.sunlabs.archsim.fleet.common.Data;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 *
 * @author mh193489
 */
public class TestTokenAccumulator extends TestCase {
    
    /** Filename of the scipt to be run */
    
    private static String scriptFileName= "src/com/sunlabs/archsim/fleet/ships/tokenaccumulator/TestTokenAccumulatorScript.xml";
    
    private static Director director;
    
    /** indexes for the data sources in the static sequenceData array in PayloadSource */
    
    private static final String DATA_SOURCE= "DataSource";
    private static final String TOKEN_SOURCE= "TokenSource";
    private static final String DATA_SINK= "DataSink";
    private static final String TOKEN_SINK= "TokenSink";
    
    /* reference to the DataSink SHIP for result inspection */
    private DataSink dataSink;
    private DataSink tokenSink;
    
    protected void setUp() {
        
        // for sum test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(2));
        PayloadSource.add(DATA_SOURCE, new Data(3));
        PayloadSource.add(DATA_SOURCE, new Data(5));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        // for resest test
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        // for overflow test
        PayloadSource.add(DATA_SOURCE, new Data(TokenAccumulatorDefaults.OVERFLOW));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        // for reset after overflow test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        // for underflow test
        PayloadSource.add(DATA_SOURCE, new Data(TokenAccumulatorDefaults.UNDERFLOW));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        // for reset after underflow test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        PayloadSource.add(TOKEN_SOURCE, new Data(true));
        
        
        // for invalid token test
        PayloadSource.add(TOKEN_SOURCE, new Data(false));
        
        // for waiting for output token test
        PayloadSource.add(DATA_SOURCE, new Data(false));
        // ... do not feed token and test for no response
        
        
        // initialize the tests
        director= Director.getInstance();
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(scriptFileName);
        
        Runner r= new Runner();
        
        r.perform(ops);
        
        /* get the reference to current DataSink SHIP instance */
        dataSink= (DataSink)Entities.find(DATA_SINK);
        tokenSink= (DataSink)Entities.find(TOKEN_SINK);
        
    } // setUp
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        director.masterClear();
    }
    
    public static Test suite() {
        return new TestSuite(TestTokenAccumulator.class);
    }
    
    public static void main(String[] args) {
        // run the tests
        junit.textui.TestRunner.run(TestTokenAccumulator.class);
        
    } // main
    
    public void testTestSetup() {
        // data sink
        assertNotNull(dataSink);
        assertNotNull(dataSink.receivedData);
        for (Iterator<Data> i= dataSink.receivedData.iterator(); i.hasNext();) {
            // check if data is not null
            Data data= i.next();
            assertNotNull(data);
            // check if data value is not null, in case data is valid
            if (data.valid) {
                assertNotNull(data.value);
            }
        }
        
    } 
    
    /* Tests for the data interface */
    
    public void testSum() {
        assertEquals( ((Integer)dataSink.receivedData.get(0).value).intValue(), 11);
    }
    
    public void testReset() {
        // check if the second data output is 0, which was produced after reset
        assertTrue(dataSink.receivedData.get(1).valid);
        assertEquals( ((Integer)dataSink.receivedData.get(1).value).intValue(), 0);
    }
    
    public void testOverflow() {
        assertFalse(dataSink.receivedData.get(2).valid);
        assertEquals(dataSink.receivedData.get(2).value, TokenAccumulatorDefaults.PLUS_INFINITY);
    }
    
    public void testResetAfterOverflow() {
        assertTrue(dataSink.receivedData.get(3).valid);
        assertEquals(dataSink.receivedData.get(3).value, 1);
    }
    
    public void testUnderflow() {
        assertFalse(dataSink.receivedData.get(4).valid);
        assertEquals(dataSink.receivedData.get(4).value, TokenAccumulatorDefaults.MINUS_INFINITY);
    }
    
    public void testResetAfterUnderflow() {
        assertTrue(dataSink.receivedData.get(5).valid);
        assertEquals(dataSink.receivedData.get(5).value, 1);
    }
    
    /* Tests for the token interface */
    
    public void testValidTokenIn() {
        assertTrue(tokenSink.receivedData.get(1).valid);
    }
    
    public void testInvalidTokenIn() {
        assertFalse(tokenSink.receivedData.get(4).valid);
    }
    
    public void testWaitingForTokenOut() {
        // check if the last data item was not produced, waiting for token
        // TODO: fix data source so that it does not produce any data onece exhausted
        // (now an OOB data is produced)
        //assertTrue(dataSink.receivedData.size() == 8);
    }
    
}
