/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Jan 31, 2005
 *
 */
package com.sunlabs.archsim.tests.fifo;

import com.sunlabs.archsim.gasp.State;

/**
 * @author ib27688
 *
 */
public class FifoFactory {
	
	// private constructor - prevent objects from being instantiated
	private FifoFactory() {};
	
	public static Fifo please(String name, int n ) {
		Fifo fifo= new Fifo(name);
		
		// make the sink and the source
		FifoSink snk= new FifoSink("sink");
		snk.setParent(fifo);
		FifoSource src= new FifoSource("source");
		src.setParent(fifo);
		
		// add the sink and the source to the fifo
		fifo.setSink(snk);
		fifo.setSource(src);
		
		//make terminals and attach them to the source
		FifoOutput dataOut= new FifoOutput("dataOut");
		dataOut.setParent(fifo);
		FifoInput ackDataOut= new FifoInput("ackDataOut", 1);
		ackDataOut.setParent(fifo);
		src.attach(dataOut);
		src.attach(ackDataOut);
		
		// create states, terminals, and actions that form the fifo and
		// connect them together
		State dataState;
		State ackState;
		FifoInput dataIn;
		FifoOutput ackDataIn;		
		for (int i= 0; i< n; i++) {
			// make the state and connect it to the terminals
			// connected to the previous state
			dataState= new State("dataState"+i);
			dataState.setParent(fifo);
			dataState.attach(dataOut);
			
			ackState= new State("ackState" + i);
			ackState.setParent(fifo);
			ackState.attach(ackDataOut);

			
			// make a new fifo action
			FifoAction action= new FifoAction("action_"+i);
			action.setParent(fifo);
			// make terminals that attach to the action
			dataIn= new FifoInput("dataIn", 0);
			dataIn.setParent(fifo);
			ackDataIn= new FifoOutput("ackDataIn");
			ackDataIn.setParent(fifo);
			dataOut= new FifoOutput("dataOut");
			dataOut.setParent(fifo);
			ackDataOut= new FifoInput("ackDataOut", 1);
			ackDataOut.setParent(fifo);

			// two of the terminals also attach to the state between
			// this action and the prevous action
			dataState.attach(dataIn);
			ackState.attach(ackDataIn);			
			
			// attach the terminals to the action
			action.attach(dataIn);
			action.attach(ackDataIn);
			action.attach(dataOut);
			action.attach(ackDataOut);

			// now that all of the attachments are done, 
			// set the state active
			ackState.setActive(null);		
		} // for
		
		// now make the state and the termninals that connect to the sink
		dataState= new State("dataState" + n);
		dataState.setParent(fifo);
		ackState= new State("ackState"+n);
		ackState.setParent(fifo);

				
		dataIn= new FifoInput("dataIn", 0);
		dataIn.setParent(fifo);
		ackDataIn= new FifoOutput("ackDataIn");
		ackDataIn.setParent(fifo);
		
		dataState.attach(dataOut);
		ackState.attach(ackDataOut);
		dataState.attach(dataIn);
		ackState.attach(ackDataIn);
		
		// finally, attach the last two terminals to the sink
		snk.attach(dataIn);
		snk.attach(ackDataIn);

		ackState.setActive(null);
		
		return fifo;
	}
} // FifoFactory
