package edu.berkeley.sbp.meta;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.chr.*;
import edu.berkeley.sbp.misc.*;
import edu.berkeley.sbp.bind.*;
import java.util.*;
import java.lang.annotation.*;
import java.lang.reflect.*;
import java.io.*;

// FIXME: non-static methods
public class AnnotationGrammarBindingResolver extends GrammarBindingResolver {

    private static boolean harsh = true;

    private final Class _cl;
    private final Class[] _inner;
    private final Method[] _allMethods;

    public AnnotationGrammarBindingResolver(Class c) {
        this._cl = c;
        ArrayList<Class> alc = new ArrayList<Class>();
        ArrayList<Method> alm = new ArrayList<Method>();
        add(c, alc, alm);
        this._inner = (Class[])alc.toArray(new Class[0]);
        this._allMethods = (Method[])alm.toArray(new Method[0]);
    }

    public Object repeatTag() { return new Tree.ArrayBuildingTreeFunctor<Object>(); }

    public Sequence tryResolveTag(String tag, String nonTerminalName, Element[] els, Object[] labels, boolean[] drops) {
        MetaGrammar.Production p = new MetaGrammar.Production(tag, nonTerminalName, els, labels, drops);
            if (labels != null && labels.length > 0 && "numeric".equals(labels[0]))
                System.out.println("here! ");
        for(Method m : _allMethods)
            if (new MetaGrammar.Target(m).isCompatible(p))
                return new MetaGrammar.Target(m).makeSequence(p);
        for(Class c : _inner)
            for(Constructor con : c.getConstructors())
                if (new MetaGrammar.Target(con).isCompatible(p))
                    return new MetaGrammar.Target(con).makeSequence(p);
        for(Class c : _inner)
            if (new MetaGrammar.Target(c).isCompatible(p))
                return new MetaGrammar.Target(c).makeSequence(p);
        return null;
    }
    public Sequence resolveTag(String tag, String nonTerminalName, Element[] els, Object[] labels, boolean[] drops) {
        MetaGrammar.Production p = new MetaGrammar.Production(tag, nonTerminalName, els, labels, drops);
        Sequence ret = tryResolveTag(tag, nonTerminalName, els, labels, drops);
        if (ret != null) return ret;
        String message = "could not find a Java method/class/ctor matching tag \""+tag+
            "\", nonterminal \""+nonTerminalName+"\" with " + els.length + " arguments";
        if (harsh) {
            throw new RuntimeException(message);
        } else {
            System.err.println(message);
            return Sequence.rewritingSequence(tag, els, labels, drops);
        }
    }

    // helper

    private static void add(Class cl, ArrayList<Class> alc, ArrayList<Method> alm) {
        for(Method m : cl.getDeclaredMethods())
            alm.add(m);
        for(Class c : cl.getDeclaredClasses()) {
            alc.add(c);
            add(c, alc, alm);
        }
        if (cl.getSuperclass() != Object.class)
            add(cl.getSuperclass(), alc, alm);
    }

}
